#!/usr/bin/python
# -*- coding: utf-8 -*-
# checksysreport, a tool to help dealing with gss tickets.
# Copyright 2005 Pierre Amadio 
# pamadio@redhat.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
import sys
sys.path.append('/usr/share/checksysreport')
import fnmatch
import re
import os
import string
import rhnapiwrap
import cPickle
import sysreportChecker
import rpm
import pgdb
import time
import syslog
import shutil
############################################
#         GLOBAL FUNCTIONS                 #
############################################
def die(msg):
    """
    This function does not need any documentation
    """
    print msg
    sys.exit()

def init_config():
    f=open(configFile,'r')
    for l in f.readlines():
        mandreg=re.compile("^mandatory_plugin\[(\w+)\]=(\w+) (\w+)").search(l)
        if mandreg:
            plugin_name=mandreg.group(1)
            plug_opt=mandreg.group(2)
            plug_val=mandreg.group(3)
            
            if plugin_name not in config["mandatory_plugin"]:
                config["mandatory_plugin"][plugin_name]=[]
            config["mandatory_plugin"][plugin_name].append({'key':plug_opt,'value':plug_val})
            continue
        
        reg=re.compile("^(\w+)=(\S+)?\s*$").search(l)
        if re.compile("^#").match(l):
            continue
        if re.compile("^\s+$").match(l):
            continue
        if not reg:
            die("Cannot parse configuration line:\n%s"%l)
        k=reg.group(1)
        v=reg.group(2)
        if k not in config.keys():
            die("'%s' not a valid configuration value"%k)
        if k=="mandatory_plugin":
            continue
        config[k]=v
    
    f.close()

    
def sql_escape(data):
    syslog.syslog(syslog.LOG_ERR("obsolete call to sql_escape"))
    sys.exit("Obsolete call to sql_escape")

def sql_array_to_dic(pgdb_array,field_array):
    """
    With the _pg module, it was possible to get database answer in a dictionnary.
    This does not seems possible anymore with pgdb.
    This function takes a regular pgdb.fetchall() array as a first argument.
    The second argument is the name of fields associated with the query.
    It output an array of dictionnary so each field value is associated with its name.
    """
    out=[]

    for orig_line in pgdb_array:
        dic_line={}
        for i in range(0,len(field_array)):
            dic_line[field_array[i]]=orig_line[i]
        out.append(dic_line)

    return out


def select_supported_channels():
    """Return a list of channel's label still living"""
    out=[]
    snt="select channel_label,channel_name from Channels where channel_label not in "
    snt+="(select channel_label from Channels where channel_end_of_life>'1902-03-31 00:00:00')"
    snt+=" and channel_label not in (select channel_label from unsupported_channels)"
    snt+="order by channel_arch"

    db_cursor.execute(snt)
    for res in db_cursor.fetchall():
        out.append(res[0])

    return out


def delete_channel(chan_label):
    print "deleting channel",chan_label
    for child in list_all_childs_channels(chan_label):
        delete_channel(child)

    snt="delete from Packages_Per_Channels where channel_label='%s'"%chan_label
    print snt
    db_cursor.execute(snt)
    snt="delete from Channels where channel_label='%s'"%chan_label
    print snt
    db_cursor.execute(snt)
    #Now, lets get rid of all packages not mentionned in Packages_Per_Channels anymore.
    print "orphan packages ?"
    for pid in get_orphan_packages():
        print "orphan pid=",pid
        snt="delete from Changelogs where package_id='%s'"%pid
        print snt
        db_cursor.execute(snt)
        snt="delete from Files where package_id='%s'"%pid
        print snt
        db_cursor.execute(snt)
        snt="delete from Advisory_Per_Packages where package_id='%s'"%pid
        print snt
        db_cursor.execute(snt)
        snt="delete from Packages where package_id='%s'"%pid
        print snt
        db_cursor.execute(snt)

def select_all_channels_label_in_db():
    "give a list of all existing channel_label in the local database"
    out=[]
    snt="select channel_label from Channels"
    db_cursor.execute(snt)
    for line in db_cursor.fetchall():
        out.append(line[0])

    return out

def get_orphan_packages():
    """ Return a list of package_id not references in any channels anymore"""
    out=[]

    snt="select distinct package_id from Packages_Per_Channels";
    db_cursor.execute(snt)

    #ppc is a dictionnary of package references in at list 1 channel.
    ppc={}
    for i in db_cursor.fetchall():
        ppc[str(i[0])]=True

    snt="select package_id from Packages";
    db_cursor.execute(snt)
    for i in db_cursor.fetchall():
        p_id=str(i[0])
        if not ppc.get(p_id):
            out.append(p_id)

    return out

def select_allChannelsInfo():
    """
     Return a list of supported channels with some info about the channel.
     Same as old version getData(globals.channelPackageFilename())
     key:channel_arch,channel_name,channel_label,channel_parent_label,channel_end_of_life
     
    """
    out=[]
    for i in select_supported_channels():
        snt="select * from Channels where channel_label='%s'"%i
        db_cursor.execute(snt)
        all_lines=sql_array_to_dic(db_cursor.fetchall(),\
                                       ['channel_label',\
                                            'channel_name',\
                                            'channel_parent_label',\
                                            'channel_end_of_life',\
                                            'channel_arch'])

        out.append(all_lines[0])
    return out


def select_all_package_in_channel(channel,latest=False):
    """
     Gives a list of all packages available in a given channel.
     Same as old version of globals.getData(globals.channelPackageFilename2(channel,'all'))
     Sale as old version of globals.getData(globals.channelPackageFilename2(channel,'latest')) if latest is true
     

     keys:{'package_arch_label', 'package_name', 'package_epoch', 'package_version', 'package_release', 'package_id', 'package_last_modified':
    """
    snt= "select p.package_id,p.package_arch_label,p.package_name,p.package_epoch,p.package_version,p.package_release,p.package_last_modified_date "
    snt+=" from packages as p, packages_per_channels as ppc where "
    snt+="p.package_id=ppc.package_id and ppc.channel_label='%s'"%channel
    if latest:
        snt+="and ppc.is_latest='t'"

    
    """
     package_id | package_arch_label |package_name| package_epoch|package_version|package_release|package_last_modified_date
     
    """
    out=[]

    db_cursor.execute(snt)
    raw_answer=db_cursor.fetchall()
    lines=sql_array_to_dic(raw_answer,\
                               ['package_id',\
                                    'package_arch_label',\
                                    'package_name',\
                                    'package_epoch',\
                                    'package_version',\
                                    'package_release',\
                                    'package_last_modified_date'])



    for l in lines:
        dic={}
        dic['package_last_modified']=l['package_last_modified_date']
        for k in ['package_arch_label', 'package_name', 'package_epoch', 'package_version', 'package_release', 'package_id', ]:
            dic[k]=l[k]
        out.append(dic)
    return out

def is_channel_in_db(label):
    snt="select count(*) as cnt from Channels where channel_label='"+label+"'"
    db_cursor.execute(snt)
    res=db_cursor.fetchone()
    return res[0]

def is_package_in_db(pid):
    snt="select count(*) as cnt from Packages where package_id="+str(pid)
    db_cursor.execute(snt)
    res=db_cursor.fetchone()
    if res[0]>0:
        return True
    else:
        return False

def is_advisory_in_db(adv):
    snt="select count(*) as cnt from Erratas where advisory='"+str(adv)+"'"
    db_cursor.execute(snt)
    res=db_cursor.fetchone()
    return res[0]

def is_bz_in_db(bz):
    snt="select count(*) as cnt from Bugzillas where bz_id="+str(bz)
    db_cursor.execute(snt)
    res=db_cursor.fetchone()
    return res[0]
    
def is_changelog_in_db(pid):
    snt="select count(*) as cnt from changelogs where package_id=%s"%pid
    db_cursor.execute(snt)
    res=db_cursor.fetchone()
    return res[0] 

def is_fileInfo_in_db(pid):
    snt="select count(*) as cnt from Files where package_id=%i"%pid
    db_cursor.execute(snt)
    res=db_cursor.fetchone()
    return res[0]

def list_filesInfo_in_package(pid):
    """
    return a list of information related to files belonging to a given rpm.
    Cf https://rhn.redhat.com/rpc/api/packages/list_files/
    """
    out=[]
    tmp_cursor=db.cursor()
    if not is_fileInfo_in_db(pid):
        files=rhnConnector.list_filesInfo_in_package(pid)

        #transaction seems to be started automatically.
        #tmp_cursor.execute("BEGIN")

        commitTest=True
        for file in files:
            file_linkto=file['file_linkto']
            file_type=file['file_type']
            file_md5sum=file['file_md5sum']
            file_size=file['file_size']
            file_path=file['file_path']
            file_last_modified_date=file['file_last_modified_date']
            
            snt="insert into Files (file_linkto,file_type,file_md5sum,file_size,file_path,file_last_modified_date,package_id) "
            snt+=" values "
            snt+="(%(file_linkto)s,%(file_type)s,%(file_md5sum)s,%(file_size)d,%(file_path)s,%(file_last_modified_date)s,%(package_id)d)"

            try:
                tmp_cursor.execute(snt,{'file_linkto':file_linkto,
                                       'file_type':file_type,
                                       'file_md5sum':file_md5sum,
                                       'file_size':file_size,
                                       'file_path':file_path,
                                       'file_last_modified_date':file_last_modified_date,
                                       'package_id':pid\
                                           })
            except pgdb.Error,error :
                syslog.syslog(syslog.LOG_ERR,snt)
                syslog.syslog(syslog.LOG_ERR,str(error))
                commitTest=False
            except :
                db.rollback()
                raise

            dic={}
            dic['file_linkto']=file_linkto
            dic['file_type']=file_type
            dic['file_md5sum']=file_md5sum
            dic['file_size']=file_size
            dic['file_path']=file_path
            dic['file_last_modified_date']=file_last_modified_date
            out.append(dic)
            
        if commitTest:
            db.commit()
        else:
            db.rollback()
        return out

    snt="select * from Files where package_id=%s"%pid
    tmp_cursor.execute(snt)
    res=sql_array_to_dic(tmp_cursor.fetchall(),\
                             ['file_linkto',\
                                  'file_type',\
                                  'file_md5sum',\
                                  'file_size',\
                                  'file_path',\
                                  'file_last_modified_date',\
                                  'package_id'])

    tmp_cursor.close()
    return res
    
def get_changelog(pid):
    out={}
    tmp_cursor=db.cursor()
    if not is_changelog_in_db(pid):
        data=rhnConnector.get_changelog(pid)
        #transaction are supposed to be started automatically.
        commitTest=True
        for entry in data:
            cur=data[entry]
            short_date=entry
            date=cur['date']
            changes=cur['changes']
            header=cur['header']
            dic={}

            try:
                dic['date']=date
                dic['header']=header
                dic['changes']=changes

            except TypeError,error:
                syslog.syslog(syslog.LOG_ERR,"problem with changelog for package %s"%pid)
                raise
                

            out[short_date]=dic
            
            try:
                tmp_cursor.execute("set client_encoding to unicode")

                snt="insert into Changelogs (short_date,date,changes,header,package_id) "
                snt+=" values "
                snt+="( %(short_date)d,%(date)s,%(changes)s,%(header)s,%(package_id)d )"

                tmp_cursor.execute(snt,{'short_date':int(short_date),\
                                        'date':date,\
                                        'changes':changes,\
                                        'header':header,\
                                        'package_id':int(pid)\
                                        })

            except pgdb.Error,error :
                syslog.syslog(syslog.LOG_ERR,snt)
                syslog.syslog(syslog.LOG_ERR,str(error))
                
                print "____"
                print snt
                print "change=",changes
                print error
                print "#####"
                print short_date
                print "#####"
                print "_____"
                time.sleep(1)
                commitTest=False
            except :
                db.rollback()
                raise
            

        if commitTest:
            db.commit()
        else:
            db.rollback()
            return get_changelog(pid)
        tmp_cursor.close()
        return out
    
    
    snt="select * from changelogs where package_id=%s"%pid
    tmp_cursor.execute(snt)
    dicRes=sql_array_to_dic(tmp_cursor.fetchall(),\
                             ['short_date',\
                                  'date',\
                                  'changes',\
                                  'header',\
                                  'package_id',\
                                  ])

    for cur in dicRes:
        dic={}
        dic['date']=cur['date']
        dic['header']=cur['header']
        dic['changes']=cur['changes']
        out[cur['short_date']]=dic
    
    return out
        
def diff_changelog(aPid,bPid):
    """
    Return the a list of changelog entry appearing only in
    the more recent package between rpma and rpmb
    """
    out={}
    changeA=get_changelog(aPid)
    changeB=get_changelog(bPid)

    sntA="select * from packages where package_id=%d"%int(aPid)

    db_cursor.execute(sntA)
    dicResA=sql_array_to_dic(db_cursor.fetchall(),
                            ['package_id',\
                             'package_arch_label',\
                             'package_name',\
                             'package_epoch',\
                             'package_version',\
                             'package_release',\
                             'package_last_modified_date'])

    version_A=dicResA[0]['package_version']
    release_A=dicResA[0]['package_release']
    epoch_A=dicResA[0]['package_epoch']
    if not epoch_A:
        epoch_A="None"

    sntB="select * from packages where package_id=%d"%int(bPid)
    db_cursor.execute(sntB)
    dicResB=sql_array_to_dic(db_cursor.fetchall(),
                            ['package_id',\
                             'package_arch_label',\
                             'package_name',\
                             'package_epoch',\
                             'package_version',\
                             'package_release',\
                             'package_last_modified_date'])


    version_B=dicResB[0]['package_version']
    release_B=dicResB[0]['package_release']
    epoch_B=dicResB[0]['package_epoch']
    if not epoch_B:
        epoch_B="None"



    compare=rpm.labelCompare((epoch_A,version_A,release_A),\
                             (epoch_B,version_B,release_B))

    #print "compare=",compare
    if compare>0:
        fresh=changeA
        old=changeB
    else:
        fresh=changeB
        old=changeA

    #print "old",old

    # Now there is the following:
    # fresh : the most recent changelog entry
    # old   : the most oldest changelog entry
    # There is still to find the most recent common date in both entry
    # all most recent entry in 'fresh' will not exist in 'old'

    freshDates=fresh.keys()
    oldDates=old.keys()
    commonDates=SimpleSet()

    #Let s be sure we are only talking about int

    intOldDates=[]
    for p in oldDates:
        intOldDates.append(int(p))

    
    for f in freshDates:
        if int(f) in intOldDates:
            commonDates.add(f)

    #Grmblblb, sometimes, changelogs are empty:
    # cf :
    #     change_log.pxt?pid=290557
    #     change_log.pxt?pid=312809

    if not len(freshDates):
        #There is no entry even in the latest changelog
        #print "nothing fresh"
        return out

    if not len(commonDates.data):
        #The freshest rpm's changelog do not to have a common entry 
        #with the old one.
        #cf:
        #   change_log.pxt?pid=188922
        #   change_log.pxt?pid=300582
        #print "no common date"
        return fresh

    mostRecentCommonDate=commonDates.get_max()
    for f in fresh:
        if f>mostRecentCommonDate:
            out[f]=fresh[f]
    return out


def rpm_labelCompare(left,right):
    """
    compare rpm version.
    Return 1 if left is younger than right
    Return -1 if left is older than right

    https://www.redhat.com/archives/fedora-test-list/2004-July/msg00017.html
    
    """
    if 'package_epoch' in left.keys():
        lepoch=left['package_epoch']
    else:
        lepoch="None"

    if 'package_epoch' in right.keys():
        repoch=right['package_epoch']
    else:
        repoch="None"
    

    if not lepoch or re.compile("^\s+$").search(lepoch) or lepoch==0 or lepoch==None or lepoch=="":
        lepoch="None"
    if not repoch or re.compile("^\s+$").search(repoch) or repoch==0 or repoch==None or repoch=="":
        repoch="None"

    lbCompare=rpm.labelCompare((lepoch,left['package_version'],left['package_release']),\
                     (repoch,right['package_version'],right['package_release']))

    return lbCompare


def compare_advisory_label(label1,label2):
    regsnt="^\S{4}-(\d{4}):(\d+)$"
    reg1=re.compile(regsnt).search(label1)
    reg2=re.compile(regsnt).search(label2)
        
    if reg1:
        date1=int(reg1.group(1))
        index1=int(reg1.group(2))
    else:
        syslog.syslog(syslog.LOG_ERR,"Cannot guess date and index for %s"%label1)
        return 0

    if reg2:
        date2=int(reg2.group(1))
        index2=int(reg2.group(2))
    else:
        syslog.syslog(syslog.LOG_ERR,"Cannot guess date and index for %s"%label2)
        return 0

    if date2!=date1:
        if date1<date2:
            return -1
        else:
            return 1

    if index1!=index2:
        if index1<index2:
            return -1
        else:
            return 1

    return 0


def get_rpm_infos(pid,base_channel_label):
    channels=[base_channel_label]
    for i in list_all_childs_channels(base_channel_label):
        channels.append(i)
    for chan in channels:
        for rpm in list_all_rpms_in_channel(chan):
            if int(rpm['package_id'])==int(pid):
                return rpm
            
    syslog.syslog(syslog.LOG_ERR,"No rpm find for pid="+str(pid)+" and base channel="+base_channel_label)
    die("ouch")
    return False

def get_erratas_fixed_by_rpm(pid):
    out=[]
    snt="select advisory from advisory_per_packages where package_id=%i"%pid
    db_cursor.execute(snt)
    for l in db_cursor.fetchall():
        out.append(l[0])
    return out

def get_bugs_for_errata(advisory):
    out={}
    snt="select bz.*,bpa.advisory from "
    snt+="bz_per_advisory as bpa, bugzillas as bz "
    snt+="where bz.bz_id=bpa.bz_id and bpa.advisory='%s'"%advisory

    db_cursor.execute(snt)
    dicRes=sql_array_to_dic(db_cursor.fetchall(),
                            ['bz_id',\
                                 'label',\
                                 'advisory'])
    for cur in dicRes:
        out[cur['bz_id']]=cur['label']  
    return out
    
    

def get_erratas(leftPid,rightPid,base_channel_label,pkgname):
    """
    return a list describing the erratas provided by the freshest package and
    not by the other one.
    base_channel is just here to limit the set of package where to look for.
    pkgname is the name of the package we are interested in.
    """
    out=[]
    lRpm=get_rpm_infos(leftPid,base_channel_label)
    rRpm=get_rpm_infos(rightPid,base_channel_label)

    if rpm_labelCompare(lRpm,rRpm)==0:
        """those are the same package ! """
        return out

    if rpm_labelCompare(lRpm,rRpm)>0:
        oldRpm=rRpm
        youngRpm=lRpm
    else:
        oldRpm=lRpm
        youngRpm=rRpm

    """ a list of package id that would update the old rpm (no package more recent than the other argument are taken care  of"""
    updates=[]

    """
    print "old",oldRpm
    print "young",youngRpm
    """
    for cpid in list_new_package_for(oldRpm['package_id'],base_channel_label,pkgname):
        crpm=get_rpm_infos(cpid,base_channel_label)
        #print crpm["package_id"],crpm["package_version"],crpm['package_release']
        #print "test",rpm_labelCompare(youngRpm,crpm)
        if rpm_labelCompare(youngRpm,crpm)>=0:
            updates.append(crpm)



    erratas_done=SimpleSet()
    for p in updates:
        cur={}
        cur['rpm']=p
        cur['bug_per_errata']={}
        for errata in get_erratas_fixed_by_rpm(p['package_id']):
            if erratas_done.is_member(errata):
                continue
            erratas_done.add(errata)
            cur['bug_per_errata'][errata]=[]
            bugs=get_bugs_for_errata(errata)

            for b in bugs:
                cur['bug_per_errata'][errata].append({'id':b,'desc':bugs[b]})

        if len(cur['bug_per_errata'].keys()):
            """
            Sometimes strange things happens where a similar advisory is mapped to
            2 different packages belonging to the same channel.

            xorg-x11-font-utils version 6.8.2 in x86_64 release :
            1.EL.33.0.1 (pid=430732)
            1.EL.33.0.2 (pid=431141)
            
            Both are associated to RHSA-2008:0030
            
            """
            out.append(cur)


    return out
                
                

def get_all_channels():
    if cache.has_key('all_channels'):
        return cache['all_channels']
    else:
        cache['all_channels']=select_allChannelsInfo()
    return cache['all_channels']

def list_all_childs_channels(parent_label):
    """
    return a list of all childs channels of parent_label
    """
    if cache['childs_channels'].has_key(parent_label):
        return cache['childs_channels'][parent_label]
    out=[]
    for c in get_all_channels():
        if c['channel_parent_label']==parent_label:
            out.append(c['channel_label'])
            childChilds=list_all_childs_channels(c['channel_label'])
            if(len(childChilds)):
                for f in childChilds:
                    out.append(f)

    cache['childs_channels'][parent_label]=out
    return out

def list_all_rpms_in_channel(channel_label):
    if cache['all_rpms_in_channel'].has_key(channel_label):
        return cache['all_rpms_in_channel'][channel_label]
    else:
        out=select_all_package_in_channel(channel_label)
        cache['all_rpms_in_channel'][channel_label]=out
        return out
                    
def list_new_package_for(ancestor,base_channel_label,pkgname):
    """
     Return a list of up2date rpm pid that would apply
     as an update for rpm with pid of ancestor
    """
    out=[]
    child_channels=list_all_childs_channels(base_channel_label)
    content=list_all_rpms_in_channel(base_channel_label)
    ancestorRpm=get_rpm_infos(ancestor,base_channel_label)
    for rpm in content:
        if rpm['package_name']==pkgname:
                if(rpm_labelCompare(rpm,ancestorRpm)>0):
                    out.append(rpm['package_id'])

    for chan in child_channels:
        content=select_all_package_in_channel(chan)

        for rpm in content:
            if rpm['package_name']==pkgname:
                    if(rpm_labelCompare(rpm,ancestorRpm)>0):
                        out.append(rpm['package_id'])
    return out


#The following one used to be directly in the checksysreport script.
def displayListChannels():
    channels=select_allChannelsInfo()
    for c in channels:
        print c['channel_label']+" -> "+c['channel_name']

def displayListPackages(channel):
    print "Displaying up2date packages for",channel
    latest=select_all_package_in_channel(channel,True)
   
    for p in latest:
        name=p['package_name']
        version=str(p['package_version'])
        release=str(p['package_release'])
        arch=str(p['package_arch_label'])
        print name+"-"+version+"-"+release+" ("+arch+")"

def displayListAllPackages(channel):
    print "Displaying up2date packages for",channel
    latest=select_all_package_in_channel(channel)
    
    for p in latest:
        name=p['package_name']
        version=str(p['package_version'])
        release=str(p['package_release'])
        arch=str(p['package_arch_label'])
        print name+"-"+version+"-"+release+" ("+arch+") pid="+str(p['package_id'])


def sosReportCompat(dir):
    """
    Prepare the archive if it has been generated by sosreport so it looks like
    a sysreport archive
    """
    if not os.path.isdir(dir+"/sos_logs"):
        return

    targets={"/installed-rpms":"/sos_commands/rpm.rpm_-qa_--qf_---NAME----VERS",\
             "/uname":"/sos_commands/kernel.uname_-a",\
             "/lsmod":"/sos_commands/kernel.lsmod",\
             "/rpm-Va":"/sos_commands/rpm.rpm_-Va"}
    
    for dest in targets:
        full_src=dir+"/"+targets[dest]
        full_dest=dir+"/"+dest
        if not os.path.isfile(full_dest) and os.path.isfile(full_src): 
            shutil.copyfile(full_src,full_dest)
            
# In order to detect suitable base channel, there are 3 things to detect:
# 
# osRelease:  2.1,3,4 or 5
# flavour:    es as ws desktop
# arch        ia64 i386 x86_64 ppc s390 s390x
#
# osRelease has to be known before creating an instance as there are
# subclasses according to the osRelease used.
def detect_release(path):
    """
    Try to detect the RHEL release from the sysreport located in path
    Return '2.1' '3' '4' 'unsupported' or '0' if unknown
    """
    redhatReleasePath=path+'/etc/redhat-release'
    if os.path.isfile(redhatReleasePath):
        RELEASE=open(redhatReleasePath,'r')
        out='0'
        for line in RELEASE:
            rhelreg3=re.compile(r"Red Hat Enterprise Linux .* release (\d)")
            rhelmatch3=rhelreg3.match(line)
            rhelreg2AS=re.compile("Red Hat Linux Advanced Server release 2.1.*")
            rhelmatch2AS=rhelreg2AS.match(line)
            rhelreg2ES=re.compile(r"Red Hat Enterprise Linux ES release 2.1.*")
            rhelmatch2ES=rhelreg2ES.match(line)
            rhelregDesktop=re.compile(r"Red Hat Desktop release (\d)")
            rhelmatchDesktop=rhelregDesktop.match(line)
            
            if rhelmatch2AS or rhelmatch2ES:
                RELEASE.close
                return '2.1'
            if rhelmatch3:
                RELEASE.close
                return str(rhelmatch3.group(1))
            if rhelmatchDesktop:
                RELEASE.close
                return str(rhelmatchDesktop.group(1))

        RELEASE.close
    else:
        die("no such file:"+redhatReleasePath)



def generate_report(checker,options,plugin_dic):
    """
    Sample plugin_dic:
    {'openbz': [{'value': '0', 'key': '0'}], 'hwcert': [{'value': '0', 'key': '0'}]}
    """


    out=u""

    top_hook_plugins=[]
    package_hook_plugins=[]
    bottom_hook_plugins=[]

    for pname in plugin_dic:
        if not os.path.isfile("/usr/share/checksysreport/plugins/%s.py"%pname):
            continue
        """http://www.diveintopython.org/functional_programming/dynamic_import.html"""
        plugmod=__import__(pname)
        curplugin=getattr(plugmod,pname)(pname,checker,plugin_dic[pname])
        if "top" in curplugin.hooks:
            top_hook_plugins.append(curplugin)
        if "package" in curplugin.hooks:
            package_hook_plugins.append(curplugin)
        if "bottom" in curplugin.hooks:
            bottom_hook_plugins.append(curplugin)
            
    
    obsoletes=checker.get_obsolete_rpms()
    out+="arch=%s\n"%checker.osArch
    out+="release=%s\n"%checker.osRelease
    out+="flavour=%s\n"%checker.osFlavour
    out+="base=%s\n"%checker.baseChannel
    out+="\n"

    for plugin in top_hook_plugins:
        out+=plugin.run({"hook":"top"})
        out+="\n"


    dmi_info=checker.get_dmi_system_information()
    if dmi_info:
        out+=" \n"
        #for k in dmi_info:
        #    out+="%s:%s\n"%(k,dmi_info[k])
        if "Manufacturer" in dmi_info:
            out+="Manufacturer: %s\n"%dmi_info["Manufacturer"]
        if "Product Name" in dmi_info:
            out+="Product Name: %s\n"%dmi_info["Product Name"]
    
    unsupported=checker.get_list_3rdParty_installed_rpms()
    out+="######   Unsupported rpms:   ########\n"
    for p in unsupported:
        out+="%s\n"%p

    out+="#####  up2date configuration:   #####\n"
    up2Conf=checker.get_up2date_conf()
    if not up2Conf:
        out+= "No up2date configuration file were found\n"
    else:
        for k in ('noSSLServerURL','serverURL'):
            if not up2Conf.has_key(k):
                out+="No such configuration variable:%s\n"%k
                continue
            val=up2Conf[k]
            out+="%s=%s\n"%(k,val)
            if not re.compile(r"/XMLRPC;?$").search(val):
                out+="%s should ends with /XMLRPC \n"%k

    out+="#######     Kernel info      ########\n"
    running=checker.getRunningKernel()
    unsupportedModules=0
    unsupportFLag=0
    taint=checker.kernel_is_tainted()
    if taint:
        out+="%s\n"%taint

    if running:
        out+=running['package_name']+"-"+str(running['package_version'])+'-'\
              +str(running['package_release'])+" ("+running['channel_label']+')\n'
        out+="https://rhn.redhat.com/rhn/software/packages/details/Overview.do?pid="\
              +str(running['package_id'])+"\n"
        unsupportedModules=checker.get_unsupported_kernel_modules()


            
        kwarn=checker.list_known_kernel_issues()

        if len(kwarn):
            out+="\n"
            for i in kwarn:
                out+= "*** WARNING *** %s \n"%i
            out+="\n"

        if unsupportedModules:
            out+= "Unsupported modules:\n"
            for u in unsupportedModules:
                out+="  %s\n"%u


        if os.path.isfile(checker.sysreportPath+"/rpm-Va"):
            customModules=checker.get_kernel_modules_verify()
            if len(customModules):
                out+="Customised kernel modules:\n"
                for mod in customModules:
                    pattern=r"^.*/(\S+?)\.k?o$"
                    modName=False
                    if re.compile(pattern).search(mod):
                        modName=re.compile(pattern).search(mod).group(1)
                    else:
                        out+="Not able to detect module name for: %s\n"%mod
                        continue

                    isLoaded=False
                    if modName:
                        isLoaded=checker.module_is_loaded(modName)

                    out+= "  %s"%mod
                    if isLoaded:
                        out+="             "+modName+" is loaded\n"
                    else:
                        out+="             "+modName+" does not appear as loaded\n"
            else:
                out+= "All modules appears genuine\n"



        else:
            out+="No rpm-Va: cannot check if kernel modules are genuine.\n"

        out+="\n"
        for p in package_hook_plugins:
            out+=p.run({"hook":"package","package":running['package_name']})
        out+="\n"            

        kchangelogFlag=0
        for o in obsoletes:
            if o['current']['package_name']!=running['package_name'] \
                   or o['latest']['channel_label']!=running['channel_label']:
                continue

            if running['package_id']!=o['latest']['package_id'] \
                   and running['package_release']==o['current']['package_release']:
                pname=o['current']['package_name']
                latest=pname+"-"+str(o['latest']['package_version'])+"-"\
                        +str(o['latest']['package_release'])+"-"+\
                o['latest']['package_arch_label']
                out+= "Kernel not up2date (should be "+latest+")\n"


                if rpm_labelCompare(o['latest'],running)<0:
                    out+="Current kernel seems more recent than the latest one according to rhn.\n"
                
                if kchangelogFlag:

                    out+= "kernel changelog already printed\n"

                else:

                    if not options.noChangelog:
                        kchangelogFlag=1
                        diff=checker.diff_changelog(o['latest'],running)
                        dates=diff.keys()
                        dates.sort()
                        for d in dates:
                            out+=diff[d]['date']
                            out+= "\n"

                            try:
                                out+=diff[d]['header']
                            except UnicodeDecodeError,err:
                                tmpsntbefore=diff[d]['header']
                                tmpsnt=unicode(tmpsntbefore,'utf8','ignore')
                                out+=tmpsnt
                            
                            out+="\n"

                            """
                            print "___"
                            print running
                            print o['latest']
                            print diff[d]['changes']
                            """
                            
                            try:
                                out+=diff[d]['changes']
                            except UnicodeDecodeError,err:
                                #print "Error=",err
                                tmpsntbefore=diff[d]['changes']
                                tmpsnt=unicode(tmpsntbefore,'utf8','ignore')
                                #
                                #print "new=",tmpsnt
                                out+=tmpsnt
                            out+="\n"
                            
                    advisories=checker.list_advisories(running)
                    for rhka in advisories:
                        out+="%s\n"%rhka
                        kbug=checker.get_bugs_for_errata(rhka)
                        for bz in kbug:
                            try:
                                out+= "BZ#"+str(bz)+" : "+kbug[bz]+"\n"
                            except:
                                # I do not understand string management well enough
                                # to cope with strange output. see what happen with 
                                # bz#455245 here 
                                out+= "BZ#"+str(bz)+" :\n"


    else:
        out+="Detection of kernel rpm was not possible\n"
        out+=checker.get_file('uname')+"\n"





    out+="###############################################################\n"
    out+="######                Rpms to update:                  ########\n"
    out+="###############################################################\n"
    #obsoletes=checker.get_obsolete_rpms()
    for o in obsoletes:
        out+= " \n"
        pname=o['current']['package_name']
        current=pname+"-"+str(o['current']['package_version'])+"-"\
                 +unicode(o['current']['package_release'], errors='replace')+"-"\
                 +o['current']['package_arch_label']
        latest=pname+"-"+str(o['latest']['package_version'])+"-"\
                +str(o['latest']['package_release'])+"-"\
                +o['latest']['package_arch_label']+" ("\
                +o['latest']['channel_label']+")"
        #dealing with bugzilla 152276 and 157655.
        if(o['current']['package_version']!=\
           (str(o['latest']['package_version'])+'0')):
            if options.fullChangelog:
                cosmeticPrefix="###############################################################\n"
            else:
                cosmeticPrefix=""
            out+=cosmeticPrefix+current+" should be "+latest+"\n"

            fastrackFlag=False
            if re.compile(r"fastrack").search(o['latest']['channel_label']):
                fastrackFlag=True

            if rpm_labelCompare(o['latest'],o['current'])<0:
                out+="Current package seems more recent than the latest one according to rhn.\n"

                if not 'package_epoch' in o['current'].keys():
                    o['current']['package_epoch']="None"
                out+="current: epoch='%s',version=%s,release=%s\n"%(o['current']['package_epoch'],o['current']['package_version'],unicode(o['current']['package_release'], errors='replace'))
                out+="rhn: epoch='%s',version=%s,release=%s\n"%(o['latest']['package_epoch'],o['latest']['package_version'],o['latest']['package_release'])

                


            if not o['current']['package_id']:
                out+="Current package does not seem to comes from channel %s. \n"%o['current']['channel_label']
                """
                Some times, we can reach this situation when a package (say httpd) is up to date, but also present
                as an older version in some sub channels appstk or whatevever).
                In those case, httpd will be part of get_obsolete_rpms(), and at the same time, will actually be up to 
                date.It can be interesting to display packages hook plugin output, here, before continuing the loop.
                Not putting it here, would result in not having a list of open bugs for httpd in the example above.
                """

                for p in package_hook_plugins:
                    out+=p.run({"hook":"package","package":o['current']['package_name']})
                continue
            

            if options.fullChangelog or (fastrackFlag and not options.noChangelog):
                if re.compile(r"kernel").search(pname):
                    out+= "not trying to diff changelog for kernel related rpm\n"
                    out+=" \n"
                    continue

                if not o['current']['package_id']:
                    out+="Current package has no package_id \n"
                    out+=str(o['current'])+"\n"
                    continue

                diff=checker.diff_changelog(o['latest'],o['current'])
                dates=diff.keys()
                dates.sort()
                changelogData=0
                for d in dates:
                    #print unicode(diff[d]['date'],'utf8','ignore')
                    
                    changelogData=1
                    #out+= diff[d]['date']+"\n"
                    #out+= diff[d]['header']+"\n"
                    #out+= diff[d]['changes']+"\n"
                    out+= diff[d]['date']
                    out+="\n"
                    try:
                        out+= diff[d]['header']
                    except UnicodeDecodeError,err:
                        tmpsntbefore=diff[d]['header']
                        tmpsnt=unicode(tmpsntbefore,'utf8','ignore')
                        out+=tmpsnt
                        
                    out+="\n"
                    try:
                        out+= diff[d]['changes']
                    except UnicodeDecodeError,err:
                        tmpsntbefore=diff[d]['changes']
                        tmpsnt=unicode(tmpsntbefore,'utf8','ignore')
                        out+=tmpsnt
                        
                    out+="\n"
                    
                if not changelogData:
                    #There was no data in changelog diff, a bit of debug is needed:
                    #out+= "latest=",o['latest']+"\n"
                    #out+= "current=",o['current']+"\n"
                    out+="No data in changelog diff\n"

                out +=" \n"


            if not re.compile(r"kernel").search(o['current']['package_name']):

                for p in package_hook_plugins:
                    out+=p.run({"hook":"package","package":o['current']['package_name']})
                
                advisories=checker.list_advisories(o['current'])
                for rhea in advisories:
                    out+="- "+rhea+"\n"
                    bugs=checker.get_bugs_for_errata(rhea)
                    for bz in bugs:
                        try:
                            out+="     BZ#"+str(bz)+" : "+bugs[bz]+"\n"
                        except:
                            # I do not understand string management well enough
                            # to cope with strange output. see what happen with 
                            # bz#455245 here 
                            out+="     BZ#"+str(bz)+" :\n"



    out+="\n"

    for plugin in bottom_hook_plugins:
        out+=plugin.run({"hook":"bottom"})
        out+="\n"
                                                        
    return out

#############################################################
#                        CLASSES                            #
#############################################################
#There are no Set modules for RHEL3.
#Let s implement a really simple one.
#we only need to add stuff and test existence

class SimpleSet:
    "set erzats for RHEL3"
    
    def __init__(self):
        self.data={}

    def add(self,item):
        self.data[item]=1
        
    def remove(self,item):
        self.data[item]=0

    def get_max(self):
        k=self.data.keys()
        return max(k)
        
    def is_member(self,item):
        if self.data.has_key(item):
            return self.data[item]
        else:
            return 0



class checkerPlugin:
    name=""
    checker=False
    options={}
    
    def __init__(self,name,sysreportchecker,options,hooks=["top"]):
        self.checker=sysreportchecker
        self.options=options
        self.name=name
        self.hooks=hooks

    def run(self,args={}):
        print "let's run "+self.name

        
############################################
#          GLOBALS VARIABLES               #             
############################################
#directory where to store cached data.
"""
cacheDirectory=os.environ['HOME']+"/.checksysreport"
if os.path.isdir(cacheDirectory)==False:
    os.mkdir(cacheDirectory)
"""
configFile=os.environ['HOME']+"/.checksysreportrc"

config={}
config['rhn_login']=''
config['rhn_password']=''
config['rhn_server']=''
config['sql_host']=''
config['sql_port']=''
config['sql_login']=''
config['sql_password']=''
config['sql_database']=''
config['mandatory_plugin']={}

if not os.path.isfile(configFile):
    shutil.copyfile('/usr/share/checksysreport/checksysreportrc',configFile)
    die("You need to customize %s"%configFile)
else:
    init_config()

if not config['rhn_server']:
    config['rhn_server']='https://rhn.redhat.com/rpc/api'


cache={}
cache['all_rpms_in_channel']={}
cache['childs_channels']={}


"""
Let s use pygresql now:
http://soma-wiki.wikidot.com/python-cookbook#toc44

This should make sql escaping more easy.
"""
if(config['sql_host']):
    hostsnt='%s:%s'%(config['sql_host'],config['sql_port'])
    db=pgdb.connect(host=hostsnt,database=config['sql_database'],user=config['sql_login'],password=config['sql_password'])
else:
    db=pgdb.connect(database=config['sql_database'])

db_cursor=db.cursor()

 
rhn_login=config['rhn_login']
rhn_password=config['rhn_password']
rhnConnector=rhnapiwrap.rhnapiwrap(rhn_login,rhn_password,config['rhn_server'])

syslog.openlog('checksysreport')
